/*
 * OpenPrinting Vector Printer Driver API Definitions [fsgpd.h]
 *
 * Copyright (c) 2006 Free Standards Group
 * Copyright (c) 2006 Fuji Xerox Printing Systems Co., Ltd.
 * Copyright (c) 2006 Canon Inc.
 * Copyright (c) 2003-2006 AXE Inc.
 *
 * All Rights Reserverd.
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation.
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE OPEN GROUP BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef _FSGPD_H_
#define _FSGPD_H_

/* Return Values and Error Codes */
#define FSGPD_OK		0	/* -1 for errors */
#define FSGPD_FATALERROR	-1	/* error: cannot be recovered */
#define FSGPD_BADREQUEST	-2	/* error: called where it should not be called */
#define FSGPD_BADCONTEXT	-3	/* error: invalid printer context */
#define FSGPD_NOTSUPPORTED	-4	/* error: combination of parameters are set */
					/*   which cannot be handled by driver or printer */
#define FSGPD_JOBCANCELED	-5	/* error: job has been canceled by some cause */
#define FSGPD_PARAMERROR	-6	/* error: invalid parameter */

/* Basic Types */
typedef int FSGPD_DC;			/* driver/device context */
typedef int FSGPD_RESULT;		/* return value */
typedef unsigned char FSGPD_BYTE;	/* BYTE */
typedef unsigned char FSGPD_CHAR;	/* character (string) */
typedef int FSGPD_INT;			/* integer */
typedef int FSGPD_FIX;			/* fixed integer */
typedef float FSGPD_FLOAT;		/* float */
typedef unsigned int FSGPD_FLAG;	/* flags */
typedef unsigned int FSGPD_ROP;		/* raster operation */

/* graphic elements */
typedef struct _FSGPD_POINT {
	FSGPD_FIX x, y;
} FSGPD_POINT;

typedef struct _FSGPD_RECTANGLE {
	FSGPD_POINT p0;			/* start point */
	FSGPD_POINT p1;			/* diagonal point */
} FSGPD_RECTANGLE;

typedef struct _FSGPD_ROUNDRECTANGLE {
	FSGPD_POINT p0;			/* start point */
	FSGPD_POINT p1;			/* diagonal point */
	FSGPD_FIX xellipse, yellipse;
} FSGPD_ROUNDRECTANGLE;

/* Image Formats */
typedef enum _FSGPD_IMAGEFORMAT {
	FSGPD_IFORMAT_RAW		= 0, 
	FSGPD_IFORMAT_RLE		= 1,
	FSGPD_IFORMAT_JPEG		= 2,
	FSGPD_IFORMAT_PNG		= 3
} FSGPD_IMAGEFORMAT;

/* Color Presentation */
typedef enum _FSGPD_COLORMAPPING {
	FSGPD_CMAP_DIRECT		= 0,
	FSGPD_CMAP_INDEXED		= 1
} FSGPD_COLORMAPPING;

typedef enum _FSGPD_CSPACE {
	FSGPD_CSPACE_BW			= 0,
	FSGPD_CSPACE_DEVICEGRAY		= 1,
	FSGPD_CSPACE_DEVICECMY		= 2,
	FSGPD_CSPACE_DEVICECMYK		= 3,
	FSGPD_CSPACE_DEVICERGB		= 4,
	FSGPD_CSPACE_DEVICEKRGB		= 5,
	FSGPD_CSPACE_STANDARDRGB	= 6,
	FSGPD_CSPACE_STANDARDRGB64	= 7
} FSGPD_CSPACE;

/* Fill, Paint, Clip */
typedef enum _FSGPD_FILLMODE {
	FSGPD_FILLMODE_EVENODD		= 0,
	FSGPD_FILLMODE_WINDING		= 1
} FSGPD_FILLMODE;

typedef enum _FSGPD_PAINTMODE {
	FSGPD_PAINTMODE_OPAQUE		= 0,
	FSGPD_PAINTMODE_TRANSPARENT	= 1
} FSGPD_PAINTMODE;

typedef enum _FSGPD_CLIPRULE {
	FSGPD_CLIPRULE_EVENODD		= 0,
	FSGPD_CLIPRULE_WINDING		= 1
} FSGPD_CLIPRULE;

/* Line */
typedef enum _FSGPD_LINESTYLE {
	FSGPD_LINESTYLE_SOLID		= 0,
	FSGPD_LINESTYLE_DASH		= 1
} FSGPD_LINESTYLE;

typedef enum _FSGPD_LINECAP {
	FSGPD_LINECAP_BUTT		= 0,
	FSGPD_LINECAP_ROUND		= 1,
	FSGPD_LINECAP_SQUARE		= 2
} FSGPD_LINECAP;

typedef enum _FSGPD_LINEJOIN {
	FSGPD_LINEJOIN_MITER		= 0,
	FSGPD_LINEJOIN_ROUND		= 1,
	FSGPD_LINEJOIN_BEVEL		= 2
} FSGPD_LINEJOIN;

/* Brush */
typedef enum _FSGPD_BDTYPE {
	FSGPD_BDTYPE_NORMAL		= 0
} FSGPD_BDTYPE;

typedef struct _FSGPD_BRUSHDATA {
	FSGPD_BDTYPE type;
	FSGPD_INT width, height, pitch;
#if defined(__GNUC__) && __GNUC__ <= 2
	FSGPD_BYTE data[1];
#else
	FSGPD_BYTE data[];
#endif
    
} FSGPD_BRUSHDATA;

typedef struct _FSGPD_BRUSH {
	FSGPD_CSPACE colorSpace;
	FSGPD_INT color[4];		/* aRGB quadruplet */
	FSGPD_INT xorg, yorg;		/* brush origin */
					/* ignored for fsgpdSetBgColor */
	FSGPD_BRUSHDATA *pbrush;	/* pointer to brush data */
					/* solid brush used, if NULL */
} FSGPD_BRUSH;

/* Misc. Flags */
typedef enum _FSGPD_ARCMODE {
	FSGPD_ARC			= 0,
	FSGPD_CHORD			= 1,
	FSGPD_PIE			= 2
} FSGPD_ARCMODE;

typedef enum _FSGPD_ARCDIR {
	FSGPD_CLOCKWISE			= 0,
	FSGPD_COUNTERCLOCKWISE		= 1
} FSGPD_ARCDIR;

typedef enum _FSGPD_PATHMODE {
	FSGPD_PATHCLOSE			= 0,
	FSGPD_PATHOPEN			= 1
} FSGPD_PATHMODE;

/* CTM */
typedef struct _FSGPD_CTM {
	FSGPD_FLOAT a, b, c, d, e, f;
} FSGPD_CTM;

/* Device Information and Capabilites */
typedef enum _FSGPD_QUERYINFOFLAGS {
  FSGPD_QF_DEVICERESOLUTION	= 0x00000001,
  FSGPD_QF_MEDIASIZE		= 0x00000002,
  FSGPD_QF_PAGEROTATION		= 0x00000004,
  FSGPD_QF_MEDIANUP		= 0x00000008,
  FSGPD_QF_MEDIADUPLEX		= 0x00000010,
  FSGPD_QF_MEDIASOURCE		= 0x00000020,
  FSGPD_QF_MEDIADESTINATION	= 0x00000040,
  FSGPD_QF_MEDIATYPE		= 0x00000080,
  FSGPD_QF_MEDIASIZE		= 0x00010000,	/* only for fsgpdQueryDeviceInfo */
  FSGPD_QF_PRINTREGION		= 0x00020000	/* only for fsgpdQueryDeviceInfo */
} FSGPD_QUERYINFOFLAGS;


/* Function prototype */
FSGPD_DC fsgpdOpenPrinter(
	FSGPD_INT outputFD,
	FSGPD_CHAR *printerModel,
	FSGPD_INT apiVersion[2],
	FSGPD_INT *nApiEntry,
	FSGPD_INT (**apiEntry[])());

/* API Procedure Entries */
typedef	struct _FSGPD_API_PROCS {
	FSGPD_DC     (*fsgpdOpenPrinter)(FSGPD_INT,FSGPD_CHAR*,FSGPD_INT*,FSGPD_INT*,struct _FSGPD_API_PROCS**);
	FSGPD_RESULT (*fsgpdClosePrinter)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdStartJob)(FSGPD_DC,FSGPD_CHAR*);
	FSGPD_RESULT (*fsgpdEndJob)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdAbortJob)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdStartDoc)(FSGPD_DC,FSGPD_CHAR*);
	FSGPD_RESULT (*fsgpdEndDoc)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdStartPage)(FSGPD_DC,FSGPD_CHAR*);
	FSGPD_RESULT (*fsgpdEndPage)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdQueryDeviceCapability)(FSGPD_DC,FSGPD_FLAG,FSGPD_INT,FSGPD_BYTE*);
	FSGPD_RESULT (*fsgpdQueryDeviceInfo)(FSGPD_DC,FSGPD_FLAG,FSGPD_INT,FSGPD_CHAR*);
	FSGPD_RESULT (*fsgpdResetCTM)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdSetCTM)(FSGPD_DC,FSGPD_CTM*);
	FSGPD_RESULT (*fsgpdGetCTM)(FSGPD_DC,FSGPD_CTM*);
	FSGPD_RESULT (*fsgpdInitGS)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdSaveGS)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdRestoreGS)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdQueryColorSpace)(FSGPD_DC,FSGPD_CSPACE*,FSGPD_INT*);
	FSGPD_RESULT (*fsgpdSetColorSpace)(FSGPD_DC,FSGPD_CSPACE);
	FSGPD_RESULT (*fsgpdGetColorSpace)(FSGPD_DC,FSGPD_CSPACE*);
	FSGPD_RESULT (*fsgpdQueryROP)(FSGPD_DC,FSGPD_INT*,FSGPD_ROP*);
	FSGPD_RESULT (*fsgpdSetROP)(FSGPD_DC,FSGPD_ROP);
	FSGPD_RESULT (*fsgpdGetROP)(FSGPD_DC,FSGPD_ROP*);
	FSGPD_RESULT (*fsgpdSetFillMode)(FSGPD_DC,FSGPD_FILLMODE);
	FSGPD_RESULT (*fsgpdGetFillMode)(FSGPD_DC,FSGPD_FILLMODE*);
	FSGPD_RESULT (*fsgpdSetAlphaConstant)(FSGPD_DC,FSGPD_FLOAT);
	FSGPD_RESULT (*fsgpdGetAlphaConstant)(FSGPD_DC,FSGPD_FLOAT*);
	FSGPD_RESULT (*fsgpdSetLineWidth)(FSGPD_DC,FSGPD_FIX);
	FSGPD_RESULT (*fsgpdGetLineWidth)(FSGPD_DC,FSGPD_FIX*);
	FSGPD_RESULT (*fsgpdSetLineDash)(FSGPD_DC,FSGPD_FIX*,FSGPD_INT);
	FSGPD_RESULT (*fsgpdGetLineDash)(FSGPD_DC,FSGPD_FIX*,FSGPD_INT*);
	FSGPD_RESULT (*fsgpdSetLineDashOffset)(FSGPD_DC,FSGPD_FIX);
	FSGPD_RESULT (*fsgpdGetLineDashOffset)(FSGPD_DC,FSGPD_FIX*);
	FSGPD_RESULT (*fsgpdSetLineStyle)(FSGPD_DC,FSGPD_LINESTYLE);
	FSGPD_RESULT (*fsgpdGetLineStyle)(FSGPD_DC,FSGPD_LINESTYLE*);
	FSGPD_RESULT (*fsgpdSetLineCap)(FSGPD_DC,FSGPD_LINECAP);
	FSGPD_RESULT (*fsgpdGetLineCap)(FSGPD_DC,FSGPD_LINECAP*);
	FSGPD_RESULT (*fsgpdSetLineJoin)(FSGPD_DC,FSGPD_LINEJOIN);
	FSGPD_RESULT (*fsgpdGetLineJoin)(FSGPD_DC,FSGPD_LINEJOIN*);
	FSGPD_RESULT (*fsgpdSetMiterLimit)(FSGPD_DC,FSGPD_FIX);
	FSGPD_RESULT (*fsgpdGetMiterLimit)(FSGPD_DC,FSGPD_FIX*);
	FSGPD_RESULT (*fsgpdSetPaintMode)(FSGPD_DC,FSGPD_PAINTMODE);
	FSGPD_RESULT (*fsgpdGetPaintMode)(FSGPD_DC,FSGPD_PAINTMODE*);
	FSGPD_RESULT (*fsgpdSetStrokeColor)(FSGPD_DC,FSGPD_BRUSH*);
	FSGPD_RESULT (*fsgpdSetFillColor)(FSGPD_DC,FSGPD_BRUSH*);
	FSGPD_RESULT (*fsgpdSetBgColor)(FSGPD_DC,FSGPD_BRUSH*);
	FSGPD_RESULT (*fsgpdNewPath)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdEndPath)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdStrokePath)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdFillPath)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdStrokeFillPath)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdSetClipPath)(FSGPD_DC,FSGPD_CLIPRULE);
	FSGPD_RESULT (*fsgpdResetClipPath)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdSetCurrentPoint)(FSGPD_DC,FSGPD_FIX,FSGPD_FIX);
	FSGPD_RESULT (*fsgpdLinePath)(FSGPD_DC,FSGPD_PATHMODE,FSGPD_INT,FSGPD_POINT*);
	FSGPD_RESULT (*fsgpdPolygonPath)(FSGPD_DC,FSGPD_INT,FSGPD_INT*,FSGPD_POINT*);
	FSGPD_RESULT (*fsgpdRectanglePath)(FSGPD_DC,FSGPD_INT,FSGPD_RECTANGLE*);
	FSGPD_RESULT (*fsgpdRoundRectanglePath)(FSGPD_DC,FSGPD_INT,FSGPD_ROUNDRECTANGLE*);
	FSGPD_RESULT (*fsgpdBezierPath)(FSGPD_DC,FSGPD_INT,FSGPD_POINT*);
	FSGPD_RESULT (*fsgpdArcPath)(FSGPD_DC,FSGPD_ARCMODE,FSGPD_ARCDIR,FSGPD_FIX,FSGPD_FIX,FSGPD_FIX,FSGPD_FIX,FSGPD_FIX,FSGPD_FIX,FSGPD_FIX,FSGPD_FIX);
	FSGPD_RESULT (*fsgpdDrawImage)(FSGPD_DC,FSGPD_INT,FSGPD_INT,FSGPD_INT,FSGPD_INT,FSGPD_IMAGEFORMAT,FSGPD_RECTANGLE,void*);
	FSGPD_RESULT (*fsgpdStartDrawImage)(FSGPD_DC,FSGPD_INT,FSGPD_INT,FSGPD_INT,FSGPD_INT,FSGPD_IMAGEFORMAT,FSGPD_RECTANGLE);
	FSGPD_RESULT (*fsgpdTransferDrawImage)(FSGPD_DC,FSGPD_INT,void*);
	FSGPD_RESULT (*fsgpdEndDrawImage)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdStartScanline)(FSGPD_DC,FSGPD_INT);
	FSGPD_RESULT (*fsgpdScanline)(FSGPD_DC,FSGPD_INT,FSGPD_INT*);
	FSGPD_RESULT (*fsgpdEndScanline)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdStartRaster)(FSGPD_DC,FSGPD_INT);
	FSGPD_RESULT (*fsgpdTransferRasterData)(FSGPD_DC,FSGPD_INT,FSGPD_BYTE*);
	FSGPD_RESULT (*fsgpdSkipRaster)(FSGPD_DC,FSGPD_INT);
	FSGPD_RESULT (*fsgpdEndRaster)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdStartStream)(FSGPD_DC);
	FSGPD_RESULT (*fsgpdTransferStreamData)(FSGPD_DC,FSGPD_INT,void*);
	FSGPD_RESULT (*fsgpdEndStream)(FSGPD_DC);
} FSGPD_API_PROCS;

#endif /* _FSGPD_H_ */
